class Wheel
	Tex = AS::RectTexture.new(AS.load_img('gfx/wheel.png'), :blocky)
	Radius = Tex.w/2.0
  Sprite = AS::Scene::SpriteTemplateNode.new(Tex, Radius, Radius)
  	
  Mass = 3.0*Math::PI*Radius**2.0
  Moment = CP.moment_for_circle(Mass, 0.0, Radius, vec2(0.0, 0.0))

	attr_reader :body, :view_node

	def initialize(pos = vec2(0.0, 0.0))
    @body = CP::Body.new(Mass, Moment)
    @body.p = pos
    
    @shape = CP::Shape::Circle.new(@body, Radius, vec2(0.0,0.0))
		@shape.u = 10.7
		@shape.group = :buggy
		
		@view_node = AS::Scene::BodyNode.new(@body, Sprite)
	end
	
	def add_to_space(space)
    space.add_body(@body)
    space.add_shape(@shape)
	end
end

class Chassis
	Tex = AS::RectTexture.new(AS.load_img('gfx/body.png'), :blocky)
	Width, Height = Tex.w, Tex.h
  Sprite = AS::Scene::SpriteTemplateNode.new(Tex, Width/2, Height/2)
  
  Radius = Wheel::Radius
  CapOffset = Width/2.0 - Radius
	
  Mass = Width*Height*5.0
  Moment = Mass*(Width**2.0 + Height**2.0)/12.0

	attr_reader :body, :view_node
	
	def initialize(pos = vec2(0.0, 0.0))
    @body = CP::Body.new(Mass, Moment)
    @body.p = pos
    
    @shape1 = make_cap(CapOffset)
    @shape2 = make_cap(-CapOffset)
    
    @view_node = AS::Scene::BodyNode.new(@body, Sprite)
	end
	
	def make_cap(off)
    shape = CP::Shape::Circle.new(@body, Radius, vec2(off, 0.0))
		shape.u = 1.0
		shape.group = :buggy
		
		return shape
	end
	
	def add_to_space(space)
    space.add_body(@body)
    space.add_shape(@shape1)
    space.add_shape(@shape2)
	end
end

class Buggy
  attr_reader :view_node
    
  Spring = 200.0*Chassis::Mass
  Damp = 6.0*Chassis::Mass
  
  Torque = -400*Wheel::Radius*Chassis::Mass
  
  def initialize(pos)
    @view_node = AS::Scene::GroupNode.new
  
    @chassis = Chassis.new(pos)
    @view_node << @chassis.view_node
    
    off_x = 16
    off_y = -12
    
    @shock1 = vec2(-off_x, off_y)
    @wheel1 = Wheel.new(pos + vec2(-off_x, off_y))
    @joint1 = CP::Joint::Groove.new(@chassis.body, @wheel1.body, vec2(-off_x, off_y), vec2(-off_x, -off_y), vec2(0,0))
    @view_node << @wheel1.view_node

    @shock2 = vec2(off_x, off_y)
    @wheel2 = Wheel.new(pos + vec2(off_x, off_y))
    @joint2 = CP::Joint::Groove.new(@chassis.body, @wheel2.body, vec2(off_x, off_y), vec2(off_x, -off_y), vec2(0,0))
    @view_node << @wheel2.view_node
  end
  
  def add_to_space(space)
    @chassis.add_to_space(space)
    @wheel1.add_to_space(space)
    @wheel2.add_to_space(space)
    space.add_joint(@joint1)
    space.add_joint(@joint2)
  end
  
  MAX_W = 200.0
  
	def torque_wheel(body, throttle)
    return if throttle == 0.0
	
    targ_w = throttle*MAX_W
    body.t += Torque*((body.w - targ_w)/MAX_W).clamp(-1.0, 1.0)
	end

  def update(dt, throttle)
    @chassis.body.reset_forces
    @wheel1.body.reset_forces
    @wheel2.body.reset_forces
    
    flip = @chassis.body.rot.dot(vec2(1.0, 0.0)) > 0.0
    
    torque_wheel(@wheel1.body, throttle)
    torque_wheel(@wheel2.body, throttle)
		@chassis.body.t -= @wheel1.body.t + @wheel2.body.t

    coef = (flip ? 1.0 : -1.0)
    shock1 = vec2(@shock1.x, coef*@shock1.y)
    shock2 = vec2(@shock2.x, coef*@shock2.y)
    CP.damped_spring(@chassis.body, @wheel1.body, shock1, vec2(0.0, 0.0), 0.0, Spring, Damp, dt)
    CP.damped_spring(@chassis.body, @wheel2.body, shock2, vec2(0.0, 0.0), 0.0, Spring, Damp, dt)
  end
  
  def pos
    @chassis.body.p
  end
  
  def vel
    @chassis.body.v
  end
  
  def shift(offset)
    v = vec2(offset, 0.0)
    @chassis.body.p -= v
    @wheel1.body.p -= v
    @wheel2.body.p -= v
  end
end
