class AS::DisplayList
	attr_reader :list_id
	
	def self.finalizer_block(id)
		lambda{GL.DeleteLists(id, 1)}
	end
  
  def gen_list
    @list_id = *GL.GenLists(1)
    ObjectSpace.define_finalizer(self, AS::DisplayList.finalizer_block(list_id))
  end

  # Creates a display list from the block passed to the constructor.
  def initialize
    gen_list

    GL.NewList(@list_id, GL::COMPILE)
      yield
    GL.EndList
  end

  # Call the display list.
  def call
    GL.CallList(@list_id)
  end
  
  def self.render_block(mode, &block)
    new {GL.render_block(mode, &block)}
  end
	
	def self.for_copy_rect(sx, sy, sw, sh, dx, dy, dw, dh)
		sright = sx + sw
		stop   = sy + sh

		dright = dx + dw
		dtop   = dy + dh

		render_block(GL::QUADS) do
      GL.TexCoord2f(    sx,   sy); GL.Vertex2f(    dx,   dy)
      GL.TexCoord2f(sright,   sy); GL.Vertex2f(dright,   dy)
      GL.TexCoord2f(sright, stop); GL.Vertex2f(dright, dtop)
      GL.TexCoord2f(    sx, stop); GL.Vertex2f(    dx, dtop)
		end
	end

	# Create a DList for a texture.
	def self.for_sprite(tex, kx, ky)    
		w, h = [tex.w, tex.h]
		tw, th = (tex.tex_target == GL::TEXTURE_RECTANGLE_EXT) ? [w, h] : [1, 1]

		return for_copy_rect(0, 0, tw, th, -kx, -ky, w, h)
	end
end
