require 'as_gamestate.rb'
require 'as_event.rb'
require 'as_sound.rb'
#require 'as_physics.rb'
require 'as_scenegraph.rb'
require 'as_color.rb'
require 'as_glextras.rb'

module AS
  autoload(:Gamestate, 'as_gamestate.rb')
  autoload(:TickTimer, 'as_timer.rb')
  autoload(:TickCounter, 'as_timer.rb')
  autoload(:FPSCounter, 'as_timer.rb')
  autoload(:Canvas, 'as_canvas.rb')
  autoload(:InputPad, 'as_input_pad.rb')
  autoload(:Texture, 'as_texture.rb')
  autoload(:RectTexture, 'as_texture.rb')
  autoload(:MIPTexture, 'as_texture.rb')
  autoload(:Transform2D, 'as_transform2D')
  autoload(:PackedMatrix, 'as_transform2D')
  autoload(:Font, 'as_font.rb')
  autoload(:DisplayList, 'as_displaylist.rb')
#  autoload(:Color, 'as_color.rb')
#  autoload(:PackedColor, 'as_color.rb')
  
	# Load an image file. Accepts either PNG or JPEG files. PNG files are
	# loaded with pre-multiplied alpha.
	def self.load_img(filename)
		case extension = File.extname(filename)
		when '.png': AS.load_png_rgba_premult(filename)
		when '.jpg': AS.load_jpeg_rgb(filename)
		else raise "Unknown extension #{extension}"
		end
	end

  # Create a basic gamma table. See also AS#set_gamma.
  def self.create_gamma_table(min, curve, max)
    min = Float(min)
    curve = Float(curve)
    max = Float(max)
    
    arr = Array.new(256) do|i|
      val = (i/255.0)**(Math::E**-curve)
      val = val*(max - min) + min
      val.clamp(0,1)*(2**16 - 1)
    end
    
    arr.pack("S*")
  end

  # Create a gamma table that simulates a display with a lower
  # bpp. See also AS#set_gamma.
  def self.create_bitcrusher_table(bits)
    shift = 8 - bits
    div = 256 - (1<<shift)
    Array.new(256){|i| ((i>>shift)<<shift)*65535/div}.pack("S*")
  end
  
  # Print out all OpenGL errors since the last call.
  def self.print_gl_errors
    until((error = GL.GetError) == 0)
      puts GLU.ErrorString(error)
    end
  end
end
