module RecursiveBuilder
  def self.append_features(mod)
    def mod.new(*args)
      args << yield if block_given?
      return super(*args)
    end

    super
  end
end

module AS::Scene
  module SceneNode
#    def inspect(indent=0)
#      str = child.inspect(indent + 1)
#      
#      "#{"\t"*indent}#{self.class}\n#{str}"
#    end
  end
  
  class BlendNode
    include RecursiveBuilder
  end
  
  class BodyNode
    include RecursiveBuilder
  end
  
  class ColorNode
    include RecursiveBuilder
  end
  
  class GroupNode
    include RecursiveBuilder

    alias_method :orig_initialize, :initialize
    def initialize(arr=[], &block)
      orig_initialize(arr, &block)
    end
    
    def self.varargs(*nodes)
      new(nodes)
    end
  
 #   def inspect(indent=0)
 #     arr = children.map{|child| child.inspect(indent + 1)}
 #     "#{"\t"*indent}AS::Scene::GroupNode\n#{arr.join("\n")}"
 #   end
    
    # Same as group_node.children << node
    def <<(node)
      self.children << node
    end
    
    # Same as group_node.children.delete(node)
    def delete(node)
      self.children.delete(node)
    end
  end
  
  class MatrixNode
    include RecursiveBuilder
  end
  
  class TranslateNode < MatrixNode
    include RecursiveBuilder
  
    def initialize(x, y, child)
      super(GL::MODELVIEW, AS::Transform2D.translate(x, y).pack, child)
    end
  end

  class ScaleNode < MatrixNode
    include RecursiveBuilder
  
    def initialize(scale, child)
      super(GL::MODELVIEW, AS::Transform2D.scale(scale, scale).pack, child)
    end
  end

  class DisplayListNode
    include RecursiveBuilder
    
    def inspect(indent=0)
      "#{"\t"*indent}#{self.class}"
    end
    
    def self.from_render_block(mode)
      new(AS::DisplayList.render_block(mode){yield})
    end
  end
  
  class TextureNode
    include RecursiveBuilder
  end
  
  class SpriteNode
    include RecursiveBuilder
  end
  
  class RubyNode
    alias_method :old_initialize, :initialize
    
    def initialize(delegate=nil, &block)
      old_initialize(delegate || block)
    end
  end
  
	class CopyRectNode < TextureNode
		def initialize(tex, sx, sy, sw, sh, dx, dy, dw, dh)
			dlist = AS::DisplayList.for_copy_rect(sx, sy, sw, sh, dx, dy, dw, dh)
			super(tex, AS::Scene::DisplayListNode.new(dlist))
		end
	end
  
  class SpriteTemplateNode < TextureNode
    def initialize(tex, kx, ky)
      super(tex, DisplayListNode.new(AS::DisplayList.for_sprite(tex, kx, ky)))
    end
    
    def wrap
      SpriteNode.new(self)
    end
  end
end
