#puts r = rand(1000)
#srand(r)
#srand(248)

AS.open_screen(640, 480, :vbl_sync => true)

require 'perlin.rb'
require 'buggy.rb'
require 'terrain.rb'

require 'load_screen.rb'

music = AS::Music.new('Flora_Fauna.ogg', true)
music.play

FontTex = AS::RectTexture.new(AS.load_png_lum_as_alpha('gfx/font.png'), :blocky, :alpha)
Font = AS::Font.new(FontTex, 'gfx/font.metrics')

W = 320
H = 240

class MyGamestate < AS::Gamestate
	def initialize
		super
		
		@space = CP::Space.new()
		@space.resize_static_hash(50.0, 1999)
		@space.resize_active_hash(50.0, 99)
		@space.gravity = vec2(0, -900)
 	
		inf = 1.0/0.0
		static_body = CP::Body.new(inf, inf)

		@view_node = AS::Scene::BlendNode.new(nil, GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA) do
		  @scene = AS::Scene::GroupNode.new
		end

		@verts = 2000
		@length = 20000.0
		scale_x = @length/@verts

    @terrain = Terrain.new(@space)
    @scene.children << @terrain.view_node

    @buggy = Buggy.new(vec2(0.0, 100.0 + TerrainBlock.terrain_func(0.0)))
    @buggy.add_to_space(@space)
    @scene.children.unshift(@buggy.view_node)
    
		@canvas = AS::Canvas.new(W/2, H/2, W, H)		
		@HUD_canvas = AS::Canvas.new(W/2, H/2, W, H)		
		
		add_splash
		add_HUD

		action(AS::Event.for_key(:Key_f)) do|pressed|
			if pressed
				j = vec2(-@chassis.m*400, 0).rotate(@chassis.rot)
				@chassis.apply_impulse(j, vec2(0, 0).rotate(@chassis.rot))
			end
		end
		
		action(AS::Event.for_key(:Key_ESCAPE)){stop}
		
		@fps = AS::FPSCounter.new(60)
		@timer = AS::TickTimer.new(60)
	end
	
	def add_splash
		text = "Buggy Racer"
		splash_text = AS::Scene::SpriteNode.new(AS::Scene::TextNode.new(Font, text))
		splash_text.pos = vec2(-Font.string_width(text)/4.0, 50.0)
		splash_text.scale = 0.5
		
		text = "Control using the arrow keys"
		keys_text = AS::Scene::SpriteNode.new(AS::Scene::TextNode.new(Font, text))
		keys_text.pos = vec2(-Font.string_width(text)/8.0, -50.0)
		keys_text.scale = 0.25
		
    @splash = AS::Scene::BlendNode.new(nil, GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA) do
      AS::Scene::ColorNode.new(@splash_color = AS::Color::Red.pack) do
        AS::Scene::GroupNode.new [splash_text, keys_text]
      end
    end
	end
	
	def add_HUD
		@dist_text = "dist: 521.12"
		dist_text = AS::Scene::SpriteNode.new(AS::Scene::TextNode.new(Font, @dist_text))
		dist_text.pos = vec2(-150.0, -80.0)
		dist_text.scale = 0.25

		@time_text = "time: 521.12"
		time_text = AS::Scene::SpriteNode.new(AS::Scene::TextNode.new(Font, @time_text))
		time_text.pos = vec2(-150.0, -95.0)
		time_text.scale = 0.25

		@vel_text = "vel: 521.12"
		vel_text = AS::Scene::SpriteNode.new(AS::Scene::TextNode.new(Font, @vel_text))
		vel_text.pos = vec2(-150.0, -110.0)
		vel_text.scale = 0.25

    @HUD = AS::Scene::BlendNode.new(nil, GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA) do
      AS::Scene::ColorNode.new(AS::Color::White.pack) do
        AS::Scene::GroupNode.new [dist_text, time_text, vel_text]
      end
    end
	end
	
	def set_dist_text
    @dist_text.replace(sprintf("Dist: %.2f", (@buggy.pos.x + @terrain.get_offset).abs/10000.0))
	end
	
	def set_time_text
    @time_text.replace(sprintf("Time: %.2f", @timer.seconds))
	end
	
	def set_vel_text
    @vel_text.replace(sprintf("Speed: %.2f", @buggy.vel.length*3600.0/10000.0))
	end
	
	ForwKey = AS::Event::Keysyms[:Key_RIGHT]
	BackKey = AS::Event::Keysyms[:Key_LEFT]
		
	MAX_W = 200.0
	
	def torque_wheel(wheel, targ_w)
    wheel.t += @torque*((wheel.w - targ_w)/MAX_W).clamp(-1.0, 1.0)
	end
	
	def update
		@redraw = @timer.sleep
#		puts @fps if @fps.tick
		
		forw = AS::Event.key_value(ForwKey)
		back = AS::Event.key_value(BackKey)
		
    throttle = 0.0
		if forw or back
			throttle -= 1.0 if forw
			throttle += 1.0 if back
		end
		
		steps = 3
		dt = (1.0/60.0)/steps
		steps.times do
		  @buggy.update(dt, throttle)
			
			@space.step(dt)
		end
		
		if @buggy.pos.x < 0.0
		  @terrain.shift_back
		  @buggy.shift(-TerrainBlock::Width)
		end
		
		if @buggy.pos.x > TerrainBlock::Width
		  @terrain.shift_forward
		  @buggy.shift(TerrainBlock::Width)
		end
	end
	
	def draw
		GL.ClearColor(0, 0, 1, 0)
		AS.clear_screen
		
		@canvas.pos = @buggy.pos
		@canvas.set	
		@view_node.render
		
    AS::Canvas.set_bounds(-W/2, -H/2, W/2, H/2)
    
    set_dist_text
    set_time_text
    set_vel_text
    @HUD.render
    
		if @timer.ticks < 300
			alpha = 1.0 - @timer.ticks/30.0
			LoadScreen.draw_with_alpha(alpha) if(alpha > 0)

		  v = [1.0, 2.5 - Float(@timer.ticks)/120.0].min
		  @splash_color.color = AS::Color::RGB.new(1.0, 1.0, 1.0, v)
		
      @splash.render
    end
	end
end

AS.clear_color(0.0, 0.0, 0.7, 1.0)

AS::Canvas.set_bounds(-W/2, -H/2, W/2, H/2)
LoadScreen.show
sleep 1.5

state = MyGamestate.new.run
